@@ -10,7 +10,7 @@ from logging import getLogger
 from os.path import isdir, isfile, join
 
 # create support
-from conda.base.constants import DepsModifier, UpdateModifier
+from conda.base.constants import ChannelPriority, DepsModifier, UpdateModifier
 from conda.base.context import context
 from conda.cli import common as cli_common
 from conda.cli.common import (
@@ -224,6 +224,9 @@ def remove(args, parser):
 
         solver_options.append((api.SOLVER_FLAG_ALLOW_UNINSTALL, 1))
 
+        if context.channel_priority is ChannelPriority.STRICT:
+            solver_options.append((api.SOLVER_FLAG_STRICT_REPO_PRIORITY, 1))
+
         pool = api.Pool()
         repos = []
 
@@ -239,7 +242,15 @@ def remove(args, parser):
             repos.append(repo)
 
         solver = api.Solver(pool, solver_options)
-        solver.add_jobs(mamba_solve_specs, api.SOLVER_ERASE)
+
+        history = api.History(context.target_prefix)
+        history_map = history.get_requested_specs_map()
+        solver.add_jobs(
+            [ms.conda_build_form() for ms in history_map.values()],
+            api.SOLVER_USERINSTALLED,
+        )
+
+        solver.add_jobs(mamba_solve_specs, api.SOLVER_ERASE | api.SOLVER_CLEANDEPS)
         success = solver.solve()
         if not success:
             print(solver.problems_to_str())
@@ -485,13 +496,15 @@ def install(args, parser, command="install"):
 
     mamba_solve_specs = [s.__str__() for s in specs]
 
+    if context.channel_priority is ChannelPriority.STRICT:
+        solver_options.append((api.SOLVER_FLAG_STRICT_REPO_PRIORITY, 1))
+
     pool = api.Pool()
 
     repos = []
 
-    if use_mamba_experimental or context.force_reinstall:
-        prefix_data = api.PrefixData(context.target_prefix)
-        prefix_data.load()
+    prefix_data = api.PrefixData(context.target_prefix)
+    prefix_data.load()
 
     # add installed
     if use_mamba_experimental:
@@ -526,6 +539,10 @@ def install(args, parser, command="install"):
                 (api.MAMBA_FORCE_REINSTALL, context.force_reinstall),
             ]
         )
+
+        if context.update_modifier is UpdateModifier.FREEZE_INSTALLED:
+            solver.add_jobs([p for p in prefix_data.package_records], api.SOLVER_LOCK)
+
         solver.add_jobs(mamba_solve_specs, solver_task)
 
         if not context.force_reinstall:
@@ -538,7 +555,7 @@ def install(args, parser, command="install"):
         pinned_specs_info = ""
         if python_constraint:
             solver.add_pin(python_constraint)
-            pinned_specs_info += f"  - {python_constraint}"
+            pinned_specs_info += f"  - {python_constraint}\n"
 
         pinned_specs = get_pinned_specs(context.target_prefix)
         if pinned_specs:
@@ -558,7 +575,7 @@ def install(args, parser, command="install"):
 
             try:
                 final_spec = s.conda_build_form()
-                pinned_specs_info += f"  - {final_spec}"
+                pinned_specs_info += f"  - {final_spec}\n"
                 solver.add_pin(final_spec)
             except AssertionError:
                 print(
@@ -704,6 +721,8 @@ def repoquery(args, parser):
         fmt = api.QueryFormat.JSON
     elif hasattr(args, "tree") and args.tree:
         fmt = api.QueryFormat.TREE
+    elif hasattr(args, "pretty") and args.pretty:
+        fmt = api.QueryFormat.PRETTY
     else:
         fmt = api.QueryFormat.TABLE
 
@@ -776,6 +795,7 @@ Examples:
 
     package_cmds.add_argument("-p", "--platform")
     package_cmds.add_argument("--no-installed", action="store_true")
+    package_cmds.add_argument("--pretty", action="store_true")
 
     package_cmds.add_argument(
         "-a",
