@@ -7,22 +7,19 @@ import json
 import os
 import tempfile
 import urllib.parse
+from collections import OrderedDict
 
 from conda._vendor.boltons.setutils import IndexedSet
 from conda.base.constants import ChannelPriority
 from conda.base.context import context
 from conda.common.serialize import json_dump
 from conda.common.url import join_url, remove_auth, split_anaconda_token
-from conda.core.index import (
-    _supplement_index_with_system,
-    calculate_channel_urls,
-    check_whitelist,
-)
+from conda.core.index import _supplement_index_with_system, check_whitelist
 from conda.core.link import PrefixSetup, UnlinkLinkTransaction
 from conda.core.prefix_data import PrefixData
 from conda.core.solve import diff_for_unlink_link_precs
 from conda.gateways.connection.session import CondaHttpAuth
-from conda.models.channel import Channel
+from conda.models.channel import Channel as CondaChannel
 from conda.models.prefix_graph import PrefixGraph
 from conda.models.records import PackageRecord
 
@@ -45,38 +42,60 @@ def get_index(
     prefix=None,
     repodata_fn="repodata.json",
 ):
+    if isinstance(platform, str):
+        platform = [platform, "noarch"]
 
-    real_urls = calculate_channel_urls(channel_urls, prepend, platform, use_local)
-    check_whitelist(real_urls)
+    all_channels = []
+    if use_local:
+        all_channels.append("local")
+    all_channels.extend(channel_urls)
+    if prepend:
+        all_channels.extend(context.channels)
+    check_whitelist(all_channels)
+
+    # Remove duplicates but retain order
+    all_channels = list(OrderedDict.fromkeys(all_channels))
 
     dlist = api.DownloadTargetList()
 
     index = []
 
-    for url in real_urls:
-        at_count = url.count("@")
+    def fixup_channel_spec(spec):
+        at_count = spec.count("@")
         if at_count > 1:
-            first_at = url.find("@")
-            url = (
-                url[:first_at] + urllib.parse.quote(url[first_at]) + url[first_at + 1 :]
+            first_at = spec.find("@")
+            spec = (
+                spec[:first_at]
+                + urllib.parse.quote(spec[first_at])
+                + spec[first_at + 1 :]
             )
-        channel = Channel(url)
-        full_url = CondaHttpAuth.add_binstar_token(
-            channel.url(with_credentials=True) + "/" + repodata_fn
-        )
+        if platform:
+            spec = spec + "[" + ",".join(platform) + "]"
+        return spec
 
-        full_path_cache = os.path.join(
-            api.create_cache_dir(), api.cache_fn_url(full_url)
-        )
-        if channel.name:
-            channel_name = channel.name + "/" + channel.subdir
-        else:
-            channel_name = channel.url(with_credentials=False)
-        sd = api.SubdirData(channel_name, full_url, full_path_cache)
+    all_channels = list(map(fixup_channel_spec, all_channels))
 
-        sd.load()
-        index.append((sd, channel))
-        dlist.add(sd)
+    for channel in api.get_channels(all_channels):
+        for channel_platform, url in channel.platform_urls(with_credentials=True):
+            full_url = CondaHttpAuth.add_binstar_token(url + "/" + repodata_fn)
+
+            full_path_cache = os.path.join(
+                api.create_cache_dir(), api.cache_fn_url(full_url)
+            )
+            name = None
+            if channel.name:
+                name = channel.name + "/" + channel_platform
+            else:
+                name = channel.platform_url(channel_platform, with_credentials=False)
+            sd = api.SubdirData(
+                name, full_url, full_path_cache, channel_platform == "noarch"
+            )
+
+            sd.load()
+            index.append(
+                (sd, {"platform": channel_platform, "url": url, "channel": channel})
+            )
+            dlist.add(sd)
 
     is_downloaded = dlist.download(True)
 
@@ -90,7 +109,7 @@ def load_channels(
     pool,
     channels,
     repos,
-    strict_priority=None,
+    has_priority=None,
     prepend=True,
     platform=None,
     use_local=False,
@@ -106,37 +125,44 @@ def load_channels(
         use_cache=use_cache,
     )
 
-    if strict_priority is None:
-        strict_priority = context.channel_priority == ChannelPriority.STRICT
+    if has_priority is None:
+        has_priority = context.channel_priority in [
+            ChannelPriority.STRICT,
+            ChannelPriority.FLEXIBLE,
+        ]
 
     subprio_index = len(index)
-    if strict_priority:
+    if has_priority:
         # first, count unique channels
-        n_channels = len(set([channel.canonical_name for _, channel in index]))
-        current_channel = index[0][1].canonical_name
+        n_channels = len(set([entry["channel"].canonical_name for _, entry in index]))
+        current_channel = index[0][1]["channel"].canonical_name
         channel_prio = n_channels
 
-    for subdir, chan in index:
+    for subdir, entry in index:
         # add priority here
-        if strict_priority:
-            if chan.canonical_name != current_channel:
+        if has_priority:
+            if entry["channel"].canonical_name != current_channel:
                 channel_prio -= 1
-                current_channel = chan.canonical_name
+                current_channel = entry["channel"].canonical_name
             priority = channel_prio
         else:
             priority = 0
-        if strict_priority:
-            subpriority = 0 if chan.platform == "noarch" else 1
+        if has_priority:
+            subpriority = 0 if entry["platform"] == "noarch" else 1
         else:
             subpriority = subprio_index
             subprio_index -= 1
 
-        if not subdir.loaded() and chan.platform != "noarch":
+        if not subdir.loaded() and entry["platform"] != "noarch":
             # ignore non-loaded subdir if channel is != noarch
             continue
 
         if context.verbosity != 0:
-            print("Channel: {}, prio: {} : {}".format(chan, priority, subpriority))
+            print(
+                "Channel: {}, platform: {}, prio: {} : {}".format(
+                    entry["channel"], entry["platform"], priority, subpriority
+                )
+            )
             print("Cache path: ", subdir.cache_path())
 
         repo = subdir.create_repo(pool)
@@ -180,13 +206,31 @@ def init_api_context(use_mamba_experimental=False):
     api_ctx.add_pip_as_python_dependency = context.add_pip_as_python_dependency
     api_ctx.use_only_tar_bz2 = context.use_only_tar_bz2
 
+    if context.channel_priority is ChannelPriority.STRICT:
+        api_ctx.channel_priority = api.ChannelPriority.kStrict
+    elif context.channel_priority is ChannelPriority.FLEXIBLE:
+        api_ctx.channel_priority = api.ChannelPriority.kFlexible
+    elif context.channel_priority is ChannelPriority.DISABLED:
+        api_ctx.channel_priority = api.ChannelPriority.kDisabled
 
-def to_package_record_from_subjson(channel, pkg, jsn_string):
-    channel = channel
-    channel_url = channel.url(with_credentials=True)
+
+def to_conda_channel(channel, platform):
+    return CondaChannel(
+        channel.scheme,
+        channel.auth,
+        channel.location,
+        channel.token,
+        channel.name,
+        platform,
+        channel.package_filename,
+    )
+
+
+def to_package_record_from_subjson(entry, pkg, jsn_string):
+    channel_url = entry["url"]
     info = json.loads(jsn_string)
     info["fn"] = pkg
-    info["channel"] = channel
+    info["channel"] = to_conda_channel(entry["channel"], entry["platform"])
     info["url"] = join_url(channel_url, pkg)
     package_record = PackageRecord(**info)
     return package_record
@@ -247,8 +291,10 @@ def to_txn(
     final_precs = IndexedSet(prefix_data.iter_records())
 
     lookup_dict = {}
-    for _, c in index:
-        lookup_dict[c.url(with_credentials=False)] = c
+    for _, entry in index:
+        lookup_dict[
+            entry["channel"].platform_url(entry["platform"], with_credentials=False)
+        ] = entry
 
     for _, pkg in to_unlink:
         for i_rec in installed_pkg_recs:
@@ -260,7 +306,16 @@ def to_txn(
             print("No package record found!")
 
     for c, pkg, jsn_s in to_link:
-        sdir = lookup_dict[split_anaconda_token(remove_auth(c))[0]]
+        if c.startswith("file://"):
+            # The conda functions (specifically remove_auth) assume the input
+            # is a url; a file uri on windows with a drive letter messes them
+            # up.
+            key = c
+        else:
+            key = split_anaconda_token(remove_auth(c))[0]
+        if key not in lookup_dict:
+            raise ValueError("missing key {} in channels: {}".format(key, lookup_dict))
+        sdir = lookup_dict[key]
         rec = to_package_record_from_subjson(sdir, pkg, jsn_s)
         final_precs.add(rec)
         to_link_records.append(rec)
